@@ -41,6 +41,10 @@
 #include "preferences.h"
 #include "bios.h"
 
+#ifdef G_OS_WIN32
+  #include "win32util.h"
+#endif
+
 
 typedef struct _MainWindowClass MainWindowClass;
 typedef struct _MainWindowPrivate MainWindowPrivate;
@@ -71,6 +75,15 @@ struct _MainWindowPrivate {
   GtkBuilder *globui;
   gboolean show_tooltips;
   gboolean sensitive;
+  gboolean treeisfocus;
+
+  const gchar* navup;
+  const gchar* navdown;
+  const gchar* navpageup;
+  const gchar* navpagedown;
+  const gchar* navfolderup;
+  const gchar* navfolderdown;
+  const gchar* navlaunch;
 };
 
 
@@ -381,7 +394,7 @@ main_window_process_exec_emu_ended (MedProcess* sender,
 
 #ifdef G_OS_WIN32
   gchar* string = NULL;
-  gchar* wpath = g_win32_get_package_installation_directory_of_module (NULL);
+  gchar* wpath = win32_get_process_directory ();
   gchar* txt = g_strconcat (wpath, "\\stdout.txt", NULL);
 
   if (g_file_get_contents(txt, &string, NULL, NULL))
@@ -419,7 +432,7 @@ main_window_process_exec_emu_ended (MedProcess* sender,
   med_process_read_conf (priv->med_process);
   main_window_set_all_values (mw, (GtkWidget*)mw);
 
-  logbook_write_log (priv->logbook, LOGBOOK_LOG_TAB_FRONTEND, "End of execution catched\n");
+  logbook_write_log (priv->logbook, LOGBOOK_LOG_TAB_FRONTEND, "End of execution caught\n");
 }
 
 
@@ -457,14 +470,39 @@ static void
 main_window_preferences_change_theme (PreferencesWindow* sender, gint t, gpointer self)
 {
 #ifdef STATIC_BUILD
-  GtkSettings *settings = gtk_settings_get_default();
 
-  if (t == 0)
-    g_object_set(settings, "gtk-theme-name", "Windows10", NULL);
-  else if (t == 1)
-    g_object_set(settings, "gtk-theme-name", "win32", NULL);
-  else if (t == 2)
-    g_object_set(settings, "gtk-theme-name", "Adwaita", NULL);
+  GtkSettings* settings = gtk_settings_get_default();
+  char* theme;
+  gboolean dark;
+
+  switch (t)
+  {
+    case 1:
+      theme = "win32";
+      dark = FALSE;
+      break;
+
+    case 2:
+      theme = "Adwaita";
+      dark = FALSE;
+      break;
+
+    case 3:
+      theme = "Windows10";
+      dark = TRUE;
+      break;
+
+    case 4:
+      theme = "Adwaita";
+      dark = TRUE;
+      break;
+
+    default:
+      theme = "Windows10";
+      dark = FALSE;
+  }
+
+  g_object_set (settings, "gtk-theme-name", theme, "gtk-application-prefer-dark-theme", dark, NULL);
 
 #endif
 }
@@ -713,7 +751,11 @@ main_window_save_settings (MainWindow* self)
   gint theme = gtk_combo_box_get_active (GTK_COMBO_BOX(priv->preferences->change_theme));
   g_key_file_set_integer (key, "GUI", "Theme", theme);
 
-  gchar* conf_path = g_win32_get_package_installation_directory_of_module (NULL);
+  GtkAdjustment *adj = gtk_spin_button_get_adjustment (priv->preferences->change_font);
+  gint fsize = (gint)gtk_adjustment_get_value (adj);
+  g_key_file_set_integer (key, "GUI", "FontSize", fsize);
+
+  gchar* conf_path = win32_get_process_directory ();
 #else
   gchar* conf_path = g_strconcat (g_get_user_config_dir (), "/mednaffe", NULL);
 
@@ -865,7 +907,7 @@ main_window_load_settings (MainWindow* self)
   GKeyFile *key = g_key_file_new ();
 
 #ifdef G_OS_WIN32
-  gchar* dir = g_win32_get_package_installation_directory_of_module (NULL);
+  gchar* dir = win32_get_process_directory ();
   gchar* conf_path = g_strconcat (dir, "\\mednaffe.conf", NULL);
   g_free(dir);
 #else
@@ -915,9 +957,17 @@ main_window_load_settings (MainWindow* self)
   gtk_widget_show ((GtkWidget*) self);
 
 #ifdef G_OS_WIN32
+
   gint t = g_key_file_get_integer (key, "GUI", "Theme", NULL);
   gtk_combo_box_set_active (GTK_COMBO_BOX(priv->preferences->change_theme), t);
-  main_window_preferences_change_theme (NULL, t, NULL);
+
+  gint s = g_key_file_get_integer (key, "GUI", "FontSize", NULL);
+
+  if (s > 0)
+    gtk_spin_button_set_value (priv->preferences->change_font, (gdouble)s);
+  else
+    gtk_spin_button_set_value (priv->preferences->change_font, 13.0);
+
 #endif
 
   gsize size;
@@ -928,10 +978,93 @@ main_window_load_settings (MainWindow* self)
   gtk_combo_box_set_active (priv->pathbox->combo, active);
 
   g_strfreev(dirs);
+
+  priv->navup = g_key_file_get_string (key, "GUI", "MoveUp", NULL);
+  priv->navdown = g_key_file_get_string (key, "GUI", "MoveDown", NULL);
+  priv->navpageup = g_key_file_get_string (key, "GUI", "MovePageUp", NULL);
+  priv->navpagedown = g_key_file_get_string (key, "GUI", "MovePageDown", NULL);
+  priv->navfolderup = g_key_file_get_string (key, "GUI", "FolderUp", NULL);
+  priv->navfolderdown = g_key_file_get_string (key, "GUI", "FolderDown", NULL);
+  priv->navlaunch = g_key_file_get_string (key, "GUI", "LaunchGame", NULL);
+
   g_key_file_free (key);
 }
 
 
+static void
+main_window_joy_event (MedListJoy* sender,
+                       const gchar* text,
+                       const gchar* value,
+                       gpointer self)
+{
+  g_return_if_fail (self != NULL);
+  g_return_if_fail (text != NULL);
+  g_return_if_fail (value != NULL);
+
+  MainWindowPrivate* priv = main_window_get_instance_private (self);
+
+  if ((!priv->treeisfocus) || (!priv->sensitive))
+    return;
+
+  if (g_strcmp0 (value, priv->navdown) == 0)
+    paned_list_selection_nav (priv->panedlist, 1, GTK_MOVEMENT_DISPLAY_LINES);
+  else if (g_strcmp0 (value, priv->navup) == 0)
+    paned_list_selection_nav (priv->panedlist, -1, GTK_MOVEMENT_DISPLAY_LINES);
+  else if (g_strcmp0 (value, priv->navpagedown) == 0)
+    paned_list_selection_nav (priv->panedlist, 1, GTK_MOVEMENT_PAGES);
+  else if (g_strcmp0 (value, priv->navpageup) == 0)
+    paned_list_selection_nav (priv->panedlist, -1, GTK_MOVEMENT_PAGES);
+  else if (g_strcmp0 (value, priv->navfolderdown) == 0)
+    path_combo_box_move_to_item (priv->pathbox, TRUE);
+  else if (g_strcmp0 (value, priv->navfolderup) == 0)
+    path_combo_box_move_to_item (priv->pathbox, FALSE);
+  else if (g_strcmp0 (value, priv->navlaunch) == 0)
+    paned_list_request_launch (priv->panedlist);
+}
+
+
+static void
+main_window_focus_changed (GtkWidget* sender,
+                           gboolean in,
+                           gpointer self)
+{
+  g_return_if_fail (self != NULL);
+  MainWindowPrivate* priv = main_window_get_instance_private (self);
+
+  med_list_joy_enable_all (priv->listjoy, in);
+  priv->treeisfocus = in;
+}
+
+
+static void
+main_window_preferences_hidden (GtkWidget* window, gpointer self)
+{
+  g_return_if_fail (self != NULL);
+  MainWindowPrivate* priv = main_window_get_instance_private (self);
+
+  GSList* it = NULL;
+  for(it = priv->preferences->list; it != NULL; it = it->next)
+  {
+    const gchar* command = med_widget_get_command ((MedWidget*)it->data);
+
+    if (g_strcmp0 (command, "MoveDown") == 0)
+      priv->navdown = med_widget_get_value ((MedWidget*)it->data);
+    else if (g_strcmp0 (command, "MoveUp") == 0)
+      priv->navup = med_widget_get_value ((MedWidget*)it->data);
+    else if (g_strcmp0 (command, "MovePageDown") == 0)
+      priv->navpagedown = med_widget_get_value ((MedWidget*)it->data);
+    else if (g_strcmp0 (command, "MovePageUp") == 0)
+      priv->navpageup = med_widget_get_value ((MedWidget*)it->data);
+    else if (g_strcmp0 (command, "FolderDown") == 0)
+      priv->navfolderdown = med_widget_get_value ((MedWidget*)it->data);
+    else if (g_strcmp0 (command, "FolderUp") == 0)
+      priv->navfolderup = med_widget_get_value ((MedWidget*)it->data);
+    else if (g_strcmp0 (command, "LaunchGame") == 0)
+      priv->navlaunch = med_widget_get_value ((MedWidget*)it->data);
+  }
+}
+
+
 void
 main_window_start (MainWindow* self)
 {
@@ -993,6 +1126,11 @@ main_window_start (MainWindow* self)
   med_list_joy_init_list_joy (priv->listjoy);
 
   g_object_set_data ((GObject*) self, "listjoy", priv->listjoy);
+  g_object_set_data ((GObject*) priv->preferences, "listjoy", priv->listjoy);
+
+  g_signal_connect_object (priv->listjoy, "joy-event", (GCallback) main_window_joy_event, self, 0);
+  g_signal_connect_object (priv->panedlist, "focus-changed", (GCallback) main_window_focus_changed, self, 0);
+  g_signal_connect_object (priv->preferences, "hide", (GCallback) main_window_preferences_hidden, self, 0);
 
   gtk_window_set_default_icon_list (g_object_get_data (app, "icon_list"));
 
@@ -1053,6 +1191,12 @@ main_window_new (GtkApplication* app)
   g_signal_connect_object (priv->panedlist, "launched", (GCallback) main_window_paned_list_launched, self, 0);
   gtk_box_pack_start (priv->main_box, (GtkWidget*) priv->panedlist, TRUE, TRUE, 0);
 
+  GList *list = NULL;
+  list = g_list_append (list, priv->panedlist);
+  list = g_list_append (list, priv->pathbox);
+  gtk_container_set_focus_chain ((GtkContainer*) priv->main_box, list);
+  g_list_free (list);
+
   priv->preferences = preferences_window_new ((GtkWindow*) self);
   g_signal_connect_object (priv->preferences, "on-show-tips", (GCallback) main_window_preferences_show_tooltips, self, 0);
   g_signal_connect_object (priv->preferences, "on-show-filters", (GCallback) main_window_preferences_show_filters, self, 0);
@@ -1075,6 +1219,7 @@ main_window_init (MainWindow* self)
   priv->sysui = NULL;
   priv->sensitive = TRUE;
   priv->show_tooltips = TRUE;
+  priv->treeisfocus = FALSE;
 
   gtk_widget_init_template (GTK_WIDGET (self));
 }
